/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.servlet.jetty;

import java.util.ArrayList;
import java.util.List;

import org.apithefire.servlet.server.AbstractServerBuilder;
import org.apithefire.servlet.server.ConnectorBuilder;
import org.apithefire.servlet.server.ContextBuilder;
import org.apithefire.servlet.server.SecureConnectorBuilder;
import org.mortbay.jetty.handler.ContextHandlerCollection;

/**
 * A {@link JettyServer} builder.
 * 
 * @author <a href="mailto:bachew@gmail.com">Chew Boon Aik</a>
 */
public class JettyServerBuilder extends AbstractServerBuilder {

	private final List<JettyConnectorBuilder> connectorBuilders;
	private final List<JettySecureConnectorBuilder> secureConnectorBuilders;
	private final List<JettyContextBuilder> contextBuilders;

	protected JettyServerBuilder() {
		connectorBuilders = new ArrayList<JettyConnectorBuilder>();
		secureConnectorBuilders = new ArrayList<JettySecureConnectorBuilder>();
		contextBuilders = new ArrayList<JettyContextBuilder>();
	}
	
	public ConnectorBuilder addConnectorBuilder() {
		JettyConnectorBuilder builder = new JettyConnectorBuilder();
		
		connectorBuilders.add(builder);
		return builder;
	}

	public SecureConnectorBuilder addSecureConnectorBuilder() {
		JettySecureConnectorBuilder builder = new JettySecureConnectorBuilder();

		secureConnectorBuilders.add(builder);
		return builder;
	}

	public ContextBuilder addContextBuilder() {
		JettyContextBuilder builder = new JettyContextBuilder();
		
		contextBuilders.add(builder);
		return builder;
	}

	/**
	 * Builds a {@link JettyServer}.
	 * 
	 * @return the built {@link JettyServer}
	 */
	public JettyServer build() {
		org.mortbay.jetty.Server server = new org.mortbay.jetty.Server();
		
		for (JettyConnectorBuilder builder : connectorBuilders) {
			server.addConnector(builder.build());
		}
		for (JettySecureConnectorBuilder builder : secureConnectorBuilders) {
			server.addConnector(builder.build());
		}
		
		ContextHandlerCollection contexts = new ContextHandlerCollection();
		
		for (JettyContextBuilder builder : contextBuilders) {
			contexts.addHandler(builder.build());
		}
		server.setHandler(contexts);
		return new JettyServer(server);
	}
	
}
